use std::str::FromStr;
use std::{env, net::SocketAddr};

use axum::{routing::get, Router};
use log::info;
use sea_orm::{ConnectOptions, Database, DatabaseConnection};

use migration::{Migrator, MigratorTrait};

mod block_types;
mod entities;
mod routes;

#[tokio::main]
async fn main() {
    if env::var("RUST_LOG").is_err() {
        env::set_var("RUST_LOG", "info");
    }
    tracing_subscriber::fmt::init();

    dotenvy::dotenv().ok();

    info!("Starting up..");

    let db_url =
        env::var("DATABASE_URL").expect("DATABASE_URL is not set in the environment! HALTING");
    let host = env::var("HOST").unwrap_or("0.0.0.0".to_string());
    let port = env::var("PORT").unwrap_or("3621".to_string());

    let server_url = format!("{host}:{port}");

    info!("Connecting to the database...");

    let mut migration_db_opt = migration::sea_orm::ConnectOptions::new(db_url.clone());
    migration_db_opt.sqlx_logging_level(log::LevelFilter::Debug);

    info!("Checking for migrations needed...");
    let migrator_conn = migration::sea_orm::Database::connect(migration_db_opt)
        .await
        .expect("Failed to connect to the database! HALTING");
    Migrator::up(&migrator_conn, None)
        .await
        .expect("Failed to run migrations! HALTING");

    let mut db_opt = ConnectOptions::new(db_url.clone());
    db_opt.sqlx_logging_level(log::LevelFilter::Debug);

    let db_conn = Database::connect(db_opt)
        .await
        .expect("Failed to connect to the database! HALTING");

    info!("Connected to database!");

    let app_state = AppState { db_conn };

    let app = Router::new()
        .route("/", get(routes::root))
        .route("/posts", get(routes::posts::posts))
        .route("/post/*slug", get(routes::post_resolver::resolver))
        .route("/*slug", get(routes::page_resolver::resolver))
        .with_state(app_state);

    info!("Attempting to bind to address...");

    let listen_addr =
        SocketAddr::from_str(&server_url).expect("Failed to parse host and port! HALTING");

    let bind = axum::Server::try_bind(&listen_addr);

    match bind {
        Ok(bind) => {
            info!("Bound to address successfully!");
            info!("Listening for connections...");
            if let Err(e) = bind.serve(app.into_make_service()).await {
                panic!("Failed to start server! HALTING {e}")
            }
        }
        Err(e) => {
            panic!("Failed to bind! HALTING {e}")
        }
    }
}

#[derive(Clone)]
struct AppState {
    db_conn: DatabaseConnection,
}
