pub(crate) mod page_resolver;
pub(crate) mod post_resolver;
pub(crate) mod posts;

use crate::{block_types::BlockTypes, AppState};
use askama::Template;
use askama_axum::IntoResponse;
use chrono::{Datelike, Utc};

use axum::extract::State;
use axum::http::StatusCode;
use log::warn;

use crate::block_types;
use crate::entities::{prelude::*, *};
use sea_orm::*;

#[derive(Template)]
#[template(path = "page.html")]
pub(crate) struct PageTemplate {
    menu: Vec<menu_entry::Model>,
    draft: bool,
    title: String,
    description: Option<String>,
    show_title: bool,
    content_blocks: Vec<BlockTypes>,
    year: String,
    settings: settings::Model,
}

#[derive(Template)]
#[template(path = "post.html")]
pub(crate) struct PostTemplate {
    menu: Vec<menu_entry::Model>,
    draft: bool,
    title: String,
    summary: Option<String>,
    publish_date: String,
    last_updated: Option<String>,
    content_blocks: Vec<BlockTypes>,
    year: String,
    settings: settings::Model,
    featured_image: Option<image::Model>,
}

#[derive(Template)]
#[template(path = "pagenotfound.html")]
pub(crate) struct PageNotFoundTemplate {
    menu: Vec<menu_entry::Model>,
    slug: String,
    year: String,
    settings: settings::Model,
}

#[derive(Template)]
#[template(path = "postnotfound.html")]
pub(crate) struct PostNotFoundTemplate {
    menu: Vec<menu_entry::Model>,
    slug: String,
    year: String,
    settings: settings::Model,
}

#[derive(Template)]
#[template(path = "page.html")]
pub(crate) struct HomeSpecialTemplate {
    menu: Vec<menu_entry::Model>,
    draft: bool,
    title: String,
    description: Option<String>,
    show_title: bool,
    content_blocks: Vec<BlockTypes>,
    year: String,
    settings: settings::Model,
}

// Handle the special home page
pub(crate) async fn root(state: State<AppState>) -> impl IntoResponse {
    // Grab special home slug page content
    let page_meta: page::Model = match Page::find()
        .filter(page::Column::Slug.eq("home"))
        .one(&state.db_conn)
        .await
        .expect("Failed to get home page! HALTING")
    {
        Some(page_meta) => page_meta,
        None => panic!("Missing \"home\" page in database!"),
    };

    let blocks: Vec<page_block::Model> = page_meta
        .find_related(PageBlock)
        .order_by_asc(page_block::Column::Order)
        .all(&state.db_conn)
        .await
        .expect("Failed to get home page blocks! HALTING");

    let mut content_blocks: Vec<BlockTypes> = vec![];

    for block in blocks {
        content_blocks.push(match block.r#type.as_str() {
            "HR" => BlockTypes::HR,
            "PARAGRAPH" => BlockTypes::PARAGRAPH {
                text: block.content,
            },
            "MARKDOWN" => BlockTypes::MARKDOWN {
                content: block.content,
            },
            "HEADER" => {
                let deserde: block_types::Header = serde_json::from_str(block.content.as_str())
                    .expect("Incorrect HEADER block formatting");

                BlockTypes::HEADER {
                    text: deserde.text,
                    size: deserde.size,
                }
            }
            "HTML" => BlockTypes::HTML {
                content: block.content,
            },
            "IMAGE" => BlockTypes::IMAGE {
                content: {
                    Image::find_by_id(block.content)
                        .one(&state.db_conn)
                        .await
                        .expect("Failed to get image! HALTING")
                },
            },
            _ => {
                warn!("Unsupported block type! ({})", block.r#type.as_str());
                BlockTypes::UNSUPPORTED
            }
        });
    }

    let menu: Vec<menu_entry::Model> = match Menu::find()
        .filter(menu::Column::Name.eq("primary"))
        .one(&state.db_conn)
        .await
        .expect("Failed to get primary menu! HALTING")
    {
        Some(menu) => menu,
        None => panic!("Missing \"primary\" menu in database!"),
    }
    .find_related(MenuEntry)
    .order_by_asc(menu_entry::Column::Order)
    .all(&state.db_conn)
    .await
    .expect("Failed to get primary menu items!");

    let settings: settings::Model = match Settings::find()
        .filter(settings::Column::Id.eq("current"))
        .one(&state.db_conn)
        .await
        .expect("Failed to get site settings!")
    {
        Some(settings) => settings,
        None => panic!("Missing \"current\" settings in database!"),
    };

    (
        StatusCode::OK,
        HomeSpecialTemplate {
            title: page_meta.clone().title,
            description: page_meta.clone().description,
            show_title: page_meta.clone().show_title,
            content_blocks,
            year: Utc::now().date_naive().year().to_string(),
            draft: false,
            menu,
            settings,
        },
    )
}
